unit NewDisk;

{
  Inno Setup
  Copyright (C) 1998-99 Jordan Russell
  For conditions of distribution and use, see LICENSE.TXT.

  New Disk form
}

interface

uses
  WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, ExtCtrls, SysUtils, Dialogs,
  SelDir;

type
  TNewDiskForm = class(TForm)
    DiskPaintBox: TPaintBox;
    SelectDiskLabel: TLabel;
    PathLabel: TLabel;
    PathEdit: TEdit;
    BrowseButton: TButton;
    CancelButton: TButton;
    OKButton: TButton;
    procedure DiskPaintBoxPaint(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure BrowseButtonClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    Filename: string;
    DiskImage: TBitmap;
  public
    { Public declarations }
  end;

function SelectDisk (const DiskNumber: Integer; const AFilename: String; var Path: String): Boolean;

implementation

uses
  Msgs, MsgIDs, CmnFunc, CmnFunc2,
  Main;

{$R *.DFM}

const
  idDirectories: array[Boolean] of TSetupMessageID =
    (msgDirectoryOld, msgDirectoryNew);

function SelectDisk (const DiskNumber: Integer; const AFilename: String;
  var Path: String): Boolean;
begin
  with TNewDiskForm.Create(Application) do
    try
      Filename := AFilename;
      SelectDiskLabel.Caption := FmtSetupMessage(msgSelectDiskLabel, [IntToStr(DiskNumber),
        StrPas(SetupMessages[idDirectories[NewGUI]])]);
      PathEdit.Text := Path;
      MessageBeep (0);
      Result := ShowModal = mrOK;
      if Result then
        Path := ExtractFilePath(ExpandFilename(AddBackslash(PathEdit.Text) + Filename));
    finally
      Free;
    end;
end;

procedure TNewDiskForm.DiskPaintBoxPaint(Sender: TObject);
begin
  with DiskPaintBox, Canvas do
    BrushCopy (ClientRect, DiskImage, ClientRect, clBlue);
end;

procedure TNewDiskForm.FormCreate(Sender: TObject);
begin
  SetTextBuf (SetupMessages[msgChangeDiskTitle]);
  PathLabel.SetTextBuf (SetupMessages[msgPathLabel]);
  BrowseButton.SetTextBuf (SetupMessages[msgButtonBrowse]);
  OKButton.SetTextBuf (SetupMessages[msgButtonOK]);
  CancelButton.SetTextBuf (SetupMessages[msgButtonCancel]);

  DiskImage := TBitmap.Create;
  DiskImage.Handle := LoadBitmap(HInstance, 'DISKIMAGE');  {don't localize}
end;

procedure TNewDiskForm.FormDestroy(Sender: TObject);
begin
  DiskImage.Free;
end;

procedure TNewDiskForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
var
  FN: String;
begin
  case ModalResult of
    mrOK: begin
        FN := ExpandFilename(AddBackslash(Trim(PathEdit.Text)) + Filename);
        if not FileExists(FN) then begin
          CanClose := False;
          MsgBox (FmtSetupMessage(msgFileNotInDir, [Filename,
            RemoveBackslashUnlessRoot(ExtractFilePath(FN)),
            StrPas(SetupMessages[idDirectories[NewGUI]])]),
            '', mbError, MB_OK);
        end;
      end;
    mrCancel: CanClose := ExitSetupMsgBox;
  end;
end;

procedure TNewDiskForm.BrowseButtonClick(Sender: TObject);
var
  Dir: String;
begin
  Dir := Trim(PathEdit.Text);
  if SelectDir(Dir, Handle) then
    PathEdit.Text := Dir;
end;

end.
